@@ -282,13 +282,16 @@ def Run(Parameters: dict, JavaJarsPath: str=None):
 
 # ------------------------------------------------ Evaluate Detection ------------------------------------------------
 def EvaluateDetection(Prediction: str, GroundTruth: str, Overlap: float, SaveImageAs: str,
-						JavaJarsPath: str=None, JavaXms: str="4G", JavaXmx: str="8G"):
+					JavaJarsPath: str=None, JavaXms: str="4G", JavaXmx: str="8G"):
 	""" This function calls the measures.Evaluations.Detection method.
 		Args:
 			Prediction (str): .
 			GroundTruth (str): .
 			Overlap (float): .
 			SaveImageAs (str): .
+			JavaJarsPath (str): .
+			JavaXms (str): .
+			JavaXmx (str): .
 	"""
 	
 	java = open("EvaluateDetection.java", "w")
