@@ -7,7 +7,6 @@ import cv2
 import numpy as np
 import psutil
 import scipy
-import skimage
 import skimage.morphology as mm
 import skimage.registration as skReg
 
@@ -16,21 +15,7 @@ from skimage.transform import warp
 
 import FiReTiTiPyLib.CyclicIF.CyclicIF_Utils as utils
 import FiReTiTiPyLib.ImagesIO as imIO
-
-
-
-class Style:
-	BLACK = '\033[30m'
-	RED = '\033[31m'
-	GREEN = '\033[32m'
-	YELLOW = '\033[33m'
-	BLUE = '\033[34m'
-	MAGENTA = '\033[35m'
-	CYAN = '\033[36m'
-	WHITE = '\033[37m'
-	UNDERLINE = '\033[4m'
-	RESET = '\033[0m'
-
+import FiReTiTiPyLib.IO.IOColors as Colors
 
 
 
@@ -40,13 +25,14 @@ class CyclicIF_Registration:
 	"""
 
 	def __init__(self, MaxFeatures: int=2013, Reverse: bool=True, SaveImages: bool=True, Copyright: str=None,
-						Debug: bool=False, MonitorMemory: bool=False):
+						Debug: bool=False, MonitorMemory: bool=False, HistogramCutoff: float=0.95):
 		""" Simple init.
 		:param MaxFeatures: Maximum number of features to extract.
 		:param Reverse: If True the last round is used as reference, otherwise the first.
 		:param SaveImages: Save the images after registration? Useful for debuging.
 		:param Debug: Run using debeging mode?
-		:param SkipTransformation: If True, the transformations are not performed.
+		:param MonitorMemory: If True, the memory information are display between important operations.
+		:param HistogramCutoff: The histogram percentage cutoff value.
 		"""
 		self.MaxFeatures = MaxFeatures
 		self.Reverse = Reverse
@@ -54,6 +40,7 @@ class CyclicIF_Registration:
 		self.Copyright = Copyright
 		self.Debug = Debug
 		self.MonitorMemory = MonitorMemory
+		self.HistogramCutoff = HistogramCutoff
 
 		#self.Safety = 32760
 		self.Warning = 46300
@@ -108,7 +95,7 @@ class CyclicIF_Registration:
 		#os.makedirs(outputdir+"/Checks/", exist_ok=True)
 
 		for scene in Scenes:
-			print(Style.GREEN + "\n\nRegistrating scene " + scene + Style.RESET)
+			print(Colors.Colors.GREEN + "\n\nRegistrating scene " + scene + Colors.Colors.RESET)
 			startime = time.time()
 			
 			for i in range(len(self.FD)): # Reset
@@ -157,12 +144,14 @@ class CyclicIF_Registration:
 						print("Level %d done in %fs" % (level, time.time()-start))
 					if not Success:
 						if self.Debug:
-							print(Style.RED + "WARNING - Registration level " + str(level) + " failure."+Style.RESET)
+							print(Colors.Colors.RED + "WARNING - Registration level " + str(level) + " failure."
+									+ Colors.Colors.RESET)
 					else:
 						break
 				
 				if not Success:
-					print(Style.RED + "Failed to find a transformation matrix => Last chance activated!" + Style.RESET)
+					print(Colors.Colors.RED + "Failed to find a transformation matrix => Last chance activated!"
+							+ Colors.Colors.RESET)
 					sys.stdout.flush()
 					print("Failed to find a transformation matrix => Last chance activated!", file=sys.stderr)
 					sys.stderr.flush()
@@ -171,7 +160,7 @@ class CyclicIF_Registration:
 					
 					if not Success:
 						nbErrors += 1
-						print(Style.RED + "Last chance failed => Registration failed." + Style.RESET)
+						print(Colors.Colors.RED + "Last chance failed => Registration failed." + Colors.Colors.RESET)
 						sys.stdout.flush()
 						print("Last chance failed => Registration failed.", file=sys.stderr)
 						sys.stderr.flush()
@@ -180,11 +169,11 @@ class CyclicIF_Registration:
 				gc.collect()
 			
 			LenIssues = len(self.Issues)
-			print(Style.GREEN + "Scene %s done in %fs:" % (scene, time.time() - startime))
+			print(Colors.Colors.GREEN + "Scene %s done in %fs:" % (scene, time.time() - startime))
 			print(" - Issues = " + str(self.Issues))
 			print(" - Last chance%s = %d" % (("" if self.Issues[LenIssues-2] <= 1 else "s"), self.Issues[LenIssues-2]))
-			print(Style.RED + " - Error%s = %d" % (("" if self.Issues[LenIssues-1] <= 1 else "s"),
-													self.Issues[LenIssues-1]) + Style.RESET)
+			print(Colors.Colors.RED + " - Error%s = %d" % (("" if self.Issues[LenIssues-1] <= 1 else "s"),
+													self.Issues[LenIssues-1]) + Colors.Colors.RESET)
 			sys.stdout.flush()
 			gc.collect()
 		
@@ -208,7 +197,7 @@ class CyclicIF_Registration:
 				print("Memory 1: After: '" + str(self.AvailableMemory()))
 			
 			if self.TargetDesc[Level] is None:
-				print(Style.RED + "ERROR 1 - Cannot extract features from target image." + Style.RESET)
+				print(Colors.Colors.RED + "ERROR 1 - Cannot extract features from target image." + Colors.Colors.RESET)
 				sys.stdout.flush()
 				print("ERROR 1 - Cannot extract features from target image.", file=sys.stderr)
 				sys.stderr.flush()
@@ -221,7 +210,7 @@ class CyclicIF_Registration:
 			print("Memory 2: after 'detectAndCompute(Image): '" + str(self.AvailableMemory()))
 		
 		if imDesc is None:
-			print(Style.RED + "ERROR 2 - Cannot extract features from image." + Style.RESET)
+			print(Colors.Colors.RED + "ERROR 2 - Cannot extract features from image." + Colors.Colors.RESET)
 			sys.stdout.flush()
 			print("ERROR 2 - Cannot extract features from image.", file=sys.stderr)
 			sys.stderr.flush()
@@ -254,11 +243,11 @@ class CyclicIF_Registration:
 					#warped_img = cv2.warpPerspective(imRaw, H, (width, height), borderMode=cv2.BORDER_CONSTANT, borderValue=0)
 					order = 1
 					#if "_c1_" not in image: order = 0
-					warped_img = warp(imRaw, Hinv, output_shape=(height, width), order=order)
+					warped_img = warp(imRaw, Hinv, output_shape=(height, width), order=order, preserve_range=True)
 					imIO.Write(warped_img.astype(np.uint16), True, outputdir + "/" + image)
 					del imRaw
 					del warped_img
-		
+			#sys.exit(0)
 		gc.collect()
 		return True
 
@@ -266,7 +255,7 @@ class CyclicIF_Registration:
 
 	def _LastChance(self, inputdir: str, Target, Image, toTransform: list, outputdir: str) -> bool:
 		if self.Debug:
-			print(Style.GREEN + "Last chance activated." + Style.RESET)
+			print(Colors.Colors.GREEN + "Last chance activated." + Colors.Colors.RESET)
 		
 		start = time.time()
 		
@@ -359,30 +348,39 @@ class CyclicIF_Registration:
 
 	def _LoadImage(self, path: str, raw: bool, preprocess: bool):
 		print("Reading '" + str(path) + "'... ", end='')
-		image = imIO.Read(path, True, verbose=False) # Image.open(dapis[0])
+		image = imIO.Read(path, Channel="First", verbose=False) # Image.open(dapis[0])
 		print(str(image.shape) + " successfully.")
 		sys.stdout.flush()
-
+		
 		channel, height, width = image.shape
 		#if self.Safety < channel or self.Safety < height or self.Safety < width:
 		#	raise Exception("Image too big. Max accepted = " + str(self.Safety))
 		
 		if self.Warning <= channel or self.Warning <= height or self.Warning <= width:
-			print("WARNING - Not recommended to register such big images as the processing will likely generate an "
-					+ "error with Java if the number of pixels is higher than 2^31. "
-					+ "Max recommended = " + str(self.Warning), file=sys.stderr)
+			print("WARNING - Very big images might generate an error with Java if the number of pixels is bigger than "
+					+ "2^31. Max recommended = " + str(self.Warning) + "x" + str(self.Warning), file=sys.stderr)
 			sys.stderr.flush()
 		
 		if raw:
 			return image
 
-		#print("Image [%d,%d]" % (image.min(), image.max()))
 		if preprocess:
+			hist, _ = np.histogram(image, bins=65535, range=(1, 65536))
+			histsum = hist.sum()
+			
+			MaxIndex = 1
+			cutoff = float(hist[MaxIndex]) / histsum
+			while cutoff < self.HistogramCutoff:
+				MaxIndex += 1
+				if 0 < hist[MaxIndex]:
+					cutoff += float(hist[MaxIndex]) / histsum
+			
 			min = image.min()
-			A = 65535.0 / (image.max() - min)
+			A = 65535.0 / (MaxIndex - min)
 			B = -A * min
 			image = image * A + B
-
+			image[65535 < image] = 65535
+		
 		image /= 65535.0
 		image *= 255.0
 		#return np.uint8(image.squeeze())
@@ -452,7 +450,7 @@ class CyclicIF_Registration:
 				print("Execution = " + str(time.time() - start) + "s")
 		
 		if count == 0:
-			print(Style.RED + "WARNING - No valid matrix detected." + Style.RESET)
+			print(Colors.Colors.RED + "WARNING - No valid matrix detected." + Colors.Colors.RESET)
 			sys.stdout.flush()
 			print("WARNING - No valid matrix detected.", file=sys.stderr)
 			sys.stderr.flush()
@@ -478,7 +476,7 @@ class CyclicIF_Registration:
 			
 			valid = self._isValidMatrix(Result)
 			if not valid:
-				print(Style.RED + "WARNING - Invalid matrix detected." + Style.RESET)
+				print(Colors.Colors.RED + "WARNING - Invalid matrix detected." + Colors.Colors.RESET)
 				sys.stdout.flush()
 				print("WARNING - Invalid matrix detected.", file=sys.stderr)
 				sys.stderr.flush()
