@@ -18,7 +18,7 @@ from FiReTiTiPyLib import ImageTools as imTools
 
 
 
-class _SingleIterator(object):
+class SingleIterator(object):
 	
 	def __init__(self, Image, CropSize: int, BorderEffectSize: int, Iterator: bool=False, verbose=False):
 		
@@ -26,7 +26,8 @@ class _SingleIterator(object):
 		self.Width, self.Height, self.Channels, self.First = imTools.Dimensions(Image)
 		
 		if verbose:
-			print("Dimensions = " + str(self.Width) + " x " + str(self.Height) + " x " + str(self.Channels) + ", channel first=" + str(self.First))
+			print("Dimensions = " + str(self.Width) + " x " + str(self.Height) + " x " + str(self.Channels) +
+				", channel first=" + str(self.First))
 		
 		self.CropSize = CropSize
 		self.Shift = CropSize - 2*BorderEffectSize
@@ -37,6 +38,7 @@ class _SingleIterator(object):
 		
 		self.lock = threading.Lock()
 		self.PositionsGenerator = self.FlowIndex()
+		self.PosX = self.PosY = 0
 	
 	
 	def Reset(self):
@@ -54,7 +56,7 @@ class _SingleIterator(object):
 				self.PosX = 0
 				wait = False
 				
-				if Ywait == False:
+				if not Ywait:
 					self.PosY += self.Shift
 				else:
 					return #-1, -1 # raise StopIteration
@@ -67,12 +69,14 @@ class _SingleIterator(object):
 			yield self.PosX, self.PosY
 			
 			
-			if Xwait == False:
+			if not Xwait:
 				self.PosX += self.Shift
 			
 			if self.Width == self.PosX+self.CropSize:
-				if Xwait == True: Xwait = False
-				else: Xwait = True
+				if Xwait:
+					Xwait = False
+				else:
+					Xwait = True
 			
 			if self.Width < self.PosX+self.CropSize:
 				self.PosX = self.Width - self.CropSize
@@ -117,8 +121,7 @@ class Segment(object):
 	
 	
 	def __Reset__(self):
-		"..."
-		return
+		pass
 
 
 
@@ -181,7 +184,7 @@ class Segment(object):
 			if InputNormalizer is not None:
 				InputNormalizer.Normalize(image)
 
-			iter = _SingleIterator(image, CropSize, BorderEffectSize, True, verbose=self.verbose) # Iterates throughout the image.
+			iter = SingleIterator(image, CropSize, BorderEffectSize, True, verbose=self.verbose) # Iterates throughout the image.
 			coordinates = []
 			try:
 				while True : # Get all the coordinates.
@@ -373,7 +376,7 @@ class Segment(object):
 			
 			batch = numpy.ndarray(shape=(BatchSize, 3, CropSize, CropSize), dtype=numpy.float32)
 
-			iter = _SingleIterator(image, CropSize, BorderEffectSize, True, verbose=False) # Iterates throughout the image.
+			iter = SingleIterator(image, CropSize, BorderEffectSize, True, verbose=False) # Iterates throughout the image.
 			coordinates = []
 			
 			while True : # Get all the coordinates.
