@@ -3,7 +3,6 @@ import torchvision
 
 from torchvision.models.detection import MaskRCNN
 from torchvision.models.detection.backbone_utils import BackboneWithFPN, resnet_fpn_backbone
-from torchvision.models.utils import load_state_dict_from_url
 from torchvision.models.detection.faster_rcnn import FastRCNNPredictor
 from torchvision.models.detection.mask_rcnn import MaskRCNNPredictor
 from torchvision.models.detection.rpn import AnchorGenerator
@@ -64,10 +63,16 @@ def ResNet50(nbClasses: int, HiddenLayer: int = 512, Pretrained: bool = True, Pr
 																box_bg_iou_thresh=box_bg_iou_thresh,
 																box_batch_size_per_image=box_batch_size_per_image,
 																box_positive_fraction=box_positive_fraction)
-	
-	if Anchors == "Nuclei":
-		anchor_generator = AnchorGenerator(sizes=tuple([(16, 32, 64) for _ in range(5)]),
-										   aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
+	if Anchors is not None:
+		anchors = Anchors.lower()
+		if anchors == "nuclei":
+			anchor_generator = AnchorGenerator(sizes=tuple([(16, 32, 64) for _ in range(5)]),
+												aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
+		elif anchors == "axons":
+			anchor_generator = AnchorGenerator(sizes=tuple([(32, 64, 128, 256) for _ in range(5)]),
+												aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
+		else:
+			raise Exception("Unknown anchors: '" + Anchors + "'. Expected Nuclei or Axons.")
 		model.rpn.anchor_generator = anchor_generator
 		model.rpn.head = RPNHead(256, anchor_generator.num_anchors_per_location()[0])
 	
@@ -287,11 +292,13 @@ def DiceLoss_Tanimoto_Dual(Input, Target, Axis=(1, 2, 3), Smooth=1.0e-5):
 
 
 def Loss(mask_logits, proposals, gt_masks, gt_labels, mask_matched_idxs):
-	# type: (Tensor, List[Tensor], List[Tensor], List[Tensor], List[Tensor])
+	### type: Tensor, List[Tensor], List[Tensor], List[Tensor], List[Tensor]
 	""" Arguments:
-			proposals (list[BoxList])
-			mask_logits (Tensor)
-			targets (list[BoxList])
+			:param proposals (list[BoxList])
+			:param mask_logits (Tensor)
+			:param gt_masks:
+			:param gt_labels:
+			:param mask_matched_idxs:
 		Return:
 			mask_loss (Tensor): scalar tensor containing the loss
 	"""
@@ -366,26 +373,26 @@ def ResNetOld(Version: int = 50, nbClasses: int = 2, HiddenLayer: int = 512, Pre
 	fpnbackbone = BackboneWithFPN(backbone, return_layers, in_channels_list, out_channels)
 	
 	model = MaskRCNN(fpnbackbone, 91,
-					 rpn_pre_nms_top_n_train=rpn_pre_nms_top_n_train,
-					 rpn_pre_nms_top_n_test=rpn_pre_nms_top_n_test,
-					 rpn_post_nms_top_n_train=rpn_post_nms_top_n_train,
-					 rpn_post_nms_top_n_test=rpn_post_nms_top_n_test,
-					 rpn_nms_thresh=rpn_nms_thresh,
-					 rpn_fg_iou_thresh=rpn_fg_iou_thresh,
-					 rpn_bg_iou_thresh=rpn_bg_iou_thresh,
-					 rpn_batch_size_per_image=rpn_batch_size_per_image,
-					 rpn_positive_fraction=rpn_positive_fraction,
-					 box_score_thresh=box_score_thresh,
-					 box_nms_thresh=box_nms_thresh,
-					 box_detections_per_img=box_detections_per_img,
-					 box_fg_iou_thresh=box_fg_iou_thresh,
-					 box_bg_iou_thresh=box_bg_iou_thresh,
-					 box_batch_size_per_image=box_batch_size_per_image,
-					 box_positive_fraction=box_positive_fraction)
+					rpn_pre_nms_top_n_train=rpn_pre_nms_top_n_train,
+					rpn_pre_nms_top_n_test=rpn_pre_nms_top_n_test,
+					rpn_post_nms_top_n_train=rpn_post_nms_top_n_train,
+					rpn_post_nms_top_n_test=rpn_post_nms_top_n_test,
+					rpn_nms_thresh=rpn_nms_thresh,
+					rpn_fg_iou_thresh=rpn_fg_iou_thresh,
+					rpn_bg_iou_thresh=rpn_bg_iou_thresh,
+					rpn_batch_size_per_image=rpn_batch_size_per_image,
+					rpn_positive_fraction=rpn_positive_fraction,
+					box_score_thresh=box_score_thresh,
+					box_nms_thresh=box_nms_thresh,
+					box_detections_per_img=box_detections_per_img,
+					box_fg_iou_thresh=box_fg_iou_thresh,
+					box_bg_iou_thresh=box_bg_iou_thresh,
+					box_batch_size_per_image=box_batch_size_per_image,
+					box_positive_fraction=box_positive_fraction)
 	
 	if Anchors == "Nuclei":
 		anchor_generator = AnchorGenerator(sizes=tuple([(16, 32, 64) for _ in range(5)]),
-										   aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
+											aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
 		model.rpn.anchor_generator = anchor_generator
 		model.rpn.head = RPNHead(256, anchor_generator.num_anchors_per_location()[0])
 	
@@ -441,26 +448,26 @@ def ResNextOld(Version: int = 50, nbClasses: int = 2, HiddenLayer: int = 512, Pr
 	fpnbackbone = BackboneWithFPN(backbone, return_layers, in_channels_list, out_channels)
 	
 	model = MaskRCNN(fpnbackbone, 91,
-					 rpn_pre_nms_top_n_train=rpn_pre_nms_top_n_train,
-					 rpn_pre_nms_top_n_test=rpn_pre_nms_top_n_test,
-					 rpn_post_nms_top_n_train=rpn_post_nms_top_n_train,
-					 rpn_post_nms_top_n_test=rpn_post_nms_top_n_test,
-					 rpn_nms_thresh=rpn_nms_thresh,
-					 rpn_fg_iou_thresh=rpn_fg_iou_thresh,
-					 rpn_bg_iou_thresh=rpn_bg_iou_thresh,
-					 rpn_batch_size_per_image=rpn_batch_size_per_image,
-					 rpn_positive_fraction=rpn_positive_fraction,
-					 box_score_thresh=box_score_thresh,
-					 box_nms_thresh=box_nms_thresh,
-					 box_detections_per_img=box_detections_per_img,
-					 box_fg_iou_thresh=box_fg_iou_thresh,
-					 box_bg_iou_thresh=box_bg_iou_thresh,
-					 box_batch_size_per_image=box_batch_size_per_image,
-					 box_positive_fraction=box_positive_fraction)
+					rpn_pre_nms_top_n_train=rpn_pre_nms_top_n_train,
+					rpn_pre_nms_top_n_test=rpn_pre_nms_top_n_test,
+					rpn_post_nms_top_n_train=rpn_post_nms_top_n_train,
+					rpn_post_nms_top_n_test=rpn_post_nms_top_n_test,
+					rpn_nms_thresh=rpn_nms_thresh,
+					rpn_fg_iou_thresh=rpn_fg_iou_thresh,
+					rpn_bg_iou_thresh=rpn_bg_iou_thresh,
+					rpn_batch_size_per_image=rpn_batch_size_per_image,
+					rpn_positive_fraction=rpn_positive_fraction,
+					box_score_thresh=box_score_thresh,
+					box_nms_thresh=box_nms_thresh,
+					box_detections_per_img=box_detections_per_img,
+					box_fg_iou_thresh=box_fg_iou_thresh,
+					box_bg_iou_thresh=box_bg_iou_thresh,
+					box_batch_size_per_image=box_batch_size_per_image,
+					box_positive_fraction=box_positive_fraction)
 	
 	if Anchors == "Nuclei":
 		anchor_generator = AnchorGenerator(sizes=tuple([(16, 32, 64) for _ in range(5)]),
-										   aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
+											aspect_ratios=tuple([(0.5, 1.0, 1.5) for _ in range(5)]))
 		model.rpn.anchor_generator = anchor_generator
 		model.rpn.head = RPNHead(256, anchor_generator.num_anchors_per_location()[0])
 	
